const {
    encrypt,
    generateHash,
    checkHash,
} = require("../../config/auth/cryptography");
const { genereteToken } = require("../../config/auth/jwt");
const { decryptDonor } = require("../../util/cryptDonor");
const db = require("../database/db");
const nodemailer = require("nodemailer");
const crypto = require("crypto");
const path = require("path");
const fs = require("fs");
const bcrypt = require('bcrypt');
const { anoMesDia } = require("../../util/dateFormat");
const { validarCNPJ } = require("../../util/validarCNPJ");
const { validarCPF } = require("../../util/validarCPF");
const { sendMessage, sendMedia } = require("../../util/gzappy")
const { getLastId, saveLastId } = require("../../util/geradorId");

const { default: axios } = require("axios");
require("dotenv").config({ path: __dirname + "/./../../../.env" });


class funcionarioRepository {
    constructor() {
        this.db = db;
    }


    /**
     * Registra os dados de um funcionario
     * @param {*} data dados do funcionario
     * @returns mensagem referente ao resultado obtido
     */
    register(data) {
        return new Promise(async (resolve, reject) => {


            const resultado = 
            data.documento.length === 11 ? validarCPF(data.documento) :
            data.documento.length === 14 ? validarCNPJ(data.documento) : null;

            if (!resultado) return resolve({ error: "CNPJ ou CPF inexistente!", code: 203 });

            const id_vendedor_rh = await new Promise((resolve, reject) => {
              this.db.query(
                `SELECT * FROM vendedores WHERE endereco = ?`,
                [data.endereco],
                (error, results) => {
                  if (error) {
                    console.error("Erro ao buscar ao vendedor ", error);
                    return reject(error);
                  }
                  resolve(results[0].id);
                }
              );
            });

            const hashedPassword = await bcrypt.hash(data.senha, 10);

            this.db.query(
                `INSERT INTO funcionarios (nome, documento, celular, endereco, senha, id_vendedor) VALUES(?,?,?,?,?,?)`,
                [data.nome, data.documento, data.celular, data.endereco, hashedPassword, id_vendedor_rh],
                async (error, response) => {
                    if (error) {
                        console.error(error);
                        resolve({ error: "Erro ao cadastrar funcionario ", code: 203 });
                    }
                    return resolve({ success: "Funcionario cadastrado com sucesso!" });
                }
            );
        });
    }


    /**
     * Lista todos os funcionarios ordernados por status e ordem alfabetica
     * @returns lista com todos os funcionarios
     */
    listAllFuncs() {
      return new Promise((resolve, reject) => {
        try {
          this.db.query(
            `SELECT funcionarios.id, funcionarios.nome, funcionarios.celular, funcionarios.documento, 
              funcionarios.endereco, funcionarios.status, funcionarios.linha, funcionarios.auth
            FROM funcionarios
            ORDER BY FIELD(funcionarios.status, 'pendente', 'aprovado', 'recusado'), funcionarios.nome DESC`,
            [],
            (error, funcionarios) => {
              if (error) return reject(new Error(error));
              return resolve(funcionarios);
            }
          );
        } catch (error) {
          return reject(new Error(error));
        }
      });
    }


    /**
     * Realiza o login de um funcionario
     * @param {*} data dados do funcionario
     * @returns mensagem referente ao resultado obtido
     */
    login(data) {
        return new Promise(async (resolve, reject) => {


            this.db.query(
                `SELECT * FROM funcionarios WHERE documento = ? `,
                [data.documento],
                async (error, response) => {
                    if (error) {
                        console.error("Database query error: ", error);
                        return reject(new Error("Database query error"));
                    }

                    if (response.length === 0) return resolve({ error: `Funcionario não encontrado`, code: 203 });

                    const userDB = response[0];

                    const isMatch = await bcrypt.compare(data.senha, userDB.senha);

                    if (!isMatch) return resolve({ error: "Senha incorreta", code: 401 });


                    const auth = crypto.randomBytes(128).toString("HEX");
                    userDB.auth = auth;

                    this.db.query(
                        "UPDATE funcionarios SET auth=? WHERE documento=?",
                        [auth, data.documento],
                        async (updateError) => {
                            if (updateError) {
                                console.error("Database update error:", updateError);
                                return reject(new Error("Database update error"));
                            }

                            this.db.query(
                                `INSERT INTO logs (id_user, data, acao) VALUES (?,?,?)`,
                                [userDB.id, new Date(), "LOGIN Funcionario"],
                                (error, results) => {
                                    if (error) {
                                        console.error("Error inserting logs: ", error);
                                        return reject(error);
                                    }
                                    resolve();
                                }
                            );

                            try {
                                const token = await genereteToken(userDB);

                                return resolve({
                                    token: token,
                                    funcionario: {
                                        id: userDB.id,
                                        nome: userDB.nome,
                                        documento: userDB.documento,
                                        celular: userDB.celular,
                                        endereco: userDB.endereco,
                                        id_vendedor: userDB.id_vendedor,
                                        auth: auth
                                    },
                                });
                            } catch (tokenError) {
                                console.error("Token generation error: ", tokenError);
                                return reject(new Error("Token generation error"));
                            }
                        }
                    );
                }
            );
        });
    }

    /**
     * Realiza o logout do funcionario
     * @param {*} data email do funcionario
     * @returns mensagem referente ao resultado obtido
     */
    logout(auth) {
        return new Promise(async (resolve, reject) => {
            this.db.query(
                "UPDATE funcionarios set auth=null WHERE auth=?",
                [auth],
                async (error, response) => {
                    if (error) return reject(new Error(error));
                    return resolve({ success: "Logout realizado com sucesso!" });
                }
            );
        });
    }


    recover(data) {
        return new Promise((resolve, reject) => {
            db.query("SELECT email, senha FROM user WHERE id = ?", [data.id_user], async (err, result) => {
                if (err) return reject("Erro no banco de dados");

                const newHashedPassword = await bcrypt.hash(data.senha, 10);

                if (result[0].senha === newHashedPassword) return resolve({ error: "Senha negada!", code: 203 });
                if (data.senha !== data.senhaConfirmacao) return resolve({ error: "Senhas informadas não coincidem", code: 203 });


                this.db.query("UPDATE user SET senha = ? WHERE email = ?", [newHashedPassword, result[0].email], (err) => {
                    if (err) {
                        console.log(err);

                        return reject("Erro ao atualizar a senha no banco de dados");
                    }

                    const templatePath = path.join(__dirname, '../../view/senha-admin.hbs');
                    const templateHtml = fs.readFileSync(templatePath, 'utf-8');

                    const emailHtml = templateHtml.replace('{{senha}}', data.senha)

                    let transporter = nodemailer.createTransport({
                        host: 'brasapedidos.com.br',
                        port: 465,
                        secure: true,
                        auth: {
                            user: process.env.EMAIL_SENDER,
                            pass: process.env.EMAIL_PASSWORD,
                        },
                    });

                    let mailOptions = {
                        from: 'suporte@brasapedidos.com.br',
                        to: data.email,
                        subject: 'Alteração de senha',
                        html: emailHtml
                    };

                    transporter.sendMail(mailOptions, (error, info) => {
                        if (error) {
                            return console.log(error);
                        }
                        console.log('Email sent: %s', info.messageId);
                    });

                    return resolve({ success: "Nova senha enviada para o email!" });

                });
            });
        });
    }


     /**
  * Lista os dados de uma unidade
  * @returns lista com todos os dados de uma unidade
  */
  listPedidosPendentes(id, id_func) {
    return new Promise(async (resolve, reject) => {

      if (!id_func) return resolve({ error: `ID do funcionario obrigatorio`, code: 203 })

      try {

        const qtdeTotalProdutos = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT SUM(pedidos_func.quantidade) AS qtde_total FROM pedidos_func WHERE id_func = ? AND concluido = false`,
            [id_func],
            (error, results) => {
              if (error) {
                console.error("Error fetching existing order:", error);
                return reject(error);
              }
              resolve(results[0].qtde_total);

            }
          );
        });


        this.db.query(`SELECT produtos_func.nome_unidade AS nome, produtos_func.id as id_produto, pedidos_func.quantidade, produtos_func.image, produtos_func.descricao_unidade AS descricao,
                      funcionarios.id AS id_func, pedidos_func.id as id_pedido, DATE_FORMAT(pedidos_func.data, '%d/%m/%Y') AS data,
                      produtos_func.categoria
                      FROM produtos_func, pedidos_func, funcionarios
                      WHERE 
                      funcionarios.id = ? AND
                      pedidos_func.concluido = false AND
                      pedidos_func.id_produto_func = produtos_func.id AND
                      pedidos_func.id_func = funcionarios.id 
                      ORDER BY pedidos_func.data DESC`,
          [id_func ? id_func : ""],
          async (error, pedidos) => {

            if (error) {

              console.log(error);
              return reject(new Error(error));
            }

            // if (qtdeMinimaSuino > qtdeTotalProdutos || qtdeMinimaBovino > qtdeTotalProdutos) {
            //   pedidos = pedidos.map(produto => {
            //     produto.message = 'true';
            //     return produto;
            //   });
            // }

            for (const produto of pedidos) {

              // if (produto.categoria === 2) {
              //   const url = "https://brasapedidos.s3.us-east-2.amazonaws.com/" + produto.image;
              //   produto.image = url;
              // }
      
            }
            return resolve(pedidos);
          });
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

    /**
     * Lista todos os produtos do funcionario
     * @param {Int} id_func Id do funcionario
     * @returns lista com todos os produtos separado por categoria
     */
      listProdutoFunc(id, id_func) {
        return new Promise(async (resolve, reject) => {

            if (!id_func) return resolve({ error: `ID do funcionario obrigatorio`, code: 203 })
  
  
  
          try {
  
            const funcionario = await new Promise((resolve, reject) => {
              this.db.query(`SELECT * FROM funcionarios WHERE id = ?`, [id_func], 
                async (error, funcionario) => {  
                if (error) { 
                  console.log(error);
                  return reject(new Error(error));
                }
                resolve(funcionario[0]);
              });
            });
            
            // Se for FUNC 2 e 3
                   
            let linhasArray = []; 
            
            if (typeof funcionario.linha === "string") {
              linhasArray = funcionario.linha.includes(",") ? funcionario.linha.split(",") : [funcionario.linha];
            }
            
            let query = "";
            
            if (linhasArray.includes("2") && linhasArray.includes("3")) 
              query = `(produtos.categoria = 2 OR produtos.categoria = 1) AND`;
            else if (linhasArray.includes("2")) query = `produtos.categoria = 2 AND`;
            else if (linhasArray.includes("3")) query = `produtos.categoria = 1 AND`;
            
            const todos_produtos = await new Promise((resolve, reject) => {
              this.db.query(
                `SELECT * FROM produtos WHERE ${query} produtos.exclusivo = false AND produtos.excluido != true ORDER BY produtos.ordem`, 
                [],
                async (error, produtos) => {  
                  if (error) { 
                    console.log(error);
                    return reject(new Error(error));
                  }
                  resolve(produtos);
                }
              );
            });
  
            // console.log(todos_produtos);
            
            
            const produtos_especifico = await new Promise((resolve, reject) => {
              this.db.query(
                `SELECT produtos.id AS id_produto, produtos.nome, produtos.descricao, produtos.image, produtos.linha, produtos.categoria, produtos.visivel, produtos.exclusivo
                 FROM produtos, produtos_disponiveis, user
                 WHERE produtos_disponiveis.id_user = user.id
                 AND produtos_disponiveis.id_produto = produtos.id
                 AND produtos_disponiveis.id_user = ?`, 
                [id_func || ""],
                async (error, produtos_especifico) => {  
                  if (error) { 
                    console.log(error);
                    return reject(new Error(error));
                  }
                  resolve(produtos_especifico);
                }
              );
            });
            
            let todos_produtos_map = new Map(todos_produtos.map(produto => [produto.id, { ...produto, especifico: false }]));
            
            for (const produto of produtos_especifico) {
              if (todos_produtos_map.has(produto.id_produto)) {
                todos_produtos_map.get(produto.id_produto).especifico = true;
              } else {
                todos_produtos_map.set(produto.id_produto, { ...produto, especifico: true });
              }
            }
            
            const resultado_final = Array.from(todos_produtos_map.values());
            
            // this.db.query(
            //   `INSERT INTO logs (id_user, data, acao) VALUES (?,?,?)`,
            //   [id_user || null, new Date(), "Entrou na pagina de produtos"],
            //   (error) => {
            //     if (error) {
            //       console.error("Error inserting logs: ", error);
            //       return reject(error);
            //     }
            //   }
            // );
            
            const produtosAgrupados = resultado_final.reduce((acc, produto) => {
              // Modificar categorias para "brasa" e "crossaint"
              const categoria = produto.categoria === 1 ? "Brasa" : produto.categoria === 2 ? "Croissaint" : produto.categoria;
  
            //   if (produto.categoria === 2) {
            //     const url = "https://brasapedidos.s3.us-east-2.amazonaws.com/" + produto.image;
            //     produto.image = url;
            // }
  
              
              if (!acc[categoria]) acc[categoria] = {};
              
              if (!acc[categoria][produto.linha]) acc[categoria][produto.linha] = [];
              
              acc[categoria][produto.linha].push({
                id: produto.id,
                nome: produto.nome_unidade,
                descricao: produto.descricao_unidade,
                image: produto.image,
                linha: produto.linha,
                categoria: categoria, // Usando a categoria modificada
                visivel: produto.visivel,
                exclusivo: produto.exclusivo,
                especifico: produto.especifico
              });
              return acc;
            }, {});
  
            // console.log(produtosAgrupados);
            
            
            return resolve(produtosAgrupados);
            
          } catch (error) {
            console.error(error);
            
            return reject(new Error(error));
          }
        });
      }


    /**
      * Lista os dados de usuarios exetos os admins
      * @returns lista com todos os dados de um usuario
      */
    listAllUser(id) {
        return new Promise((resolve, reject) => {
            try {
                this.db.query(
                    `SELECT user.id, user.razao_social, user.documento, user.celular, user.email, user.role, user.status, user.tipo, user.aroma,
                  user.dias_pedido, user.endereco, user.qtde_minima_suino, user.qtde_minima_bovino, user.qtde_minima_cross, user.linha, user.id_vendedor,
                  user.anotacoes, DATE_FORMAT(user.data_criado, "%d/%m/%Y") AS data_criado, DATE_FORMAT(pedidos.data, "%d/%m/%Y") AS data_pedido, SUM(pedidos.quantidade) AS quantidade_total_pedidos
          FROM user
          LEFT JOIN pedidos ON pedidos.id_user = user.id
          WHERE 
          user.role != 'ADMIN'
          GROUP BY user.id
          ORDER BY FIELD(user.status, 'pendente', 'aprovado', 'recusado'), user.data_criado DESC, user.razao_social`,
                    [id ? id : ""],
                    (error, usuarios) => {
                        if (error) return reject(new Error(error));

                        const usuariosTransformados = usuarios.map(usuario => ({
                            ...usuario,
                            dias_pedido: usuario.dias_pedido ? usuario.dias_pedido.match(/Segunda|Terça|Quarta|Quinta|Sexta/g) || [] : []
                        }));

                        return resolve(usuariosTransformados);
                    }
                );
            } catch (error) {
                return reject(new Error(error));
            }
        });
    }

      /**
      * Lista os dados de um funcionario
      * @param {Int} id_func id do funcionario
      * @returns lista com todos os dados de um funcionario
      */
      listFuncionarioById(id_func) {
        return new Promise((resolve, reject) => {

            if (!id_func) return resolve({ error: `ID do funcionario obrigatorio`, code: 203 })

            try {
                this.db.query(
                    `SELECT funcionarios.id, funcionarios.nome, funcionarios.documento, funcionarios.celular, funcionarios.id_vendedor
                    FROM funcionarios
                    WHERE funcionarios.id = ?`,
                    [id_func ? id_func : ""],
                    (error, funcionario) => {
                        if (error) return reject(new Error(error));
                        return resolve(funcionario);
                    }
                );
            } catch (error) {
                return reject(new Error(error));
            }
        });
    }

/**
 * Registra um pedido se não houver no banco, caso haja no banco só incrementa a quantidade
 * @param {*} data dados do pedido
 * @returns mensagem referente ao resultado obtido
 */
  registerPedido(data) {
    return new Promise(async (resolve, reject) => {
      console.log(data);

      let insert;
      let pedidoId;


      try {
        const pedidoResults = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT * FROM pedidos_func WHERE id_func = ? AND id_produto_func = ? AND concluido = false`,
            [data.id_func, data.id_produto],
            (error, results) => {
              if (error) {
                console.error("Error fetching existing order:", error);
                return reject(error);
              }
              resolve(results);
            }
          );
        });


        console.log("Existing order: ", pedidoResults);

        let qtde = parseInt(data.quantidade) // Inicializa a quantidade com o valor da nova inserção

        if (pedidoResults.length > 0) {
          // Se o pedido já existir, atualiza a quantidade
          const existingOrder = pedidoResults[0];
          qtde += existingOrder.quantidade; // Soma a quantidade existente com a nova

          console.log("Updated quantity:", qtde);

          await new Promise((resolve, reject) => {
            this.db.query(
              `UPDATE pedidos_func SET quantidade = ? WHERE id_func = ? AND id_produto_func = ? AND concluido = false`,
              [qtde, data.id_func, data.id_produto],
              (error) => {
                if (error) {
                  console.error("Error updating order:", error);
                  return reject(error);
                }
                resolve();
              }
            );
          });
        } else {
          // Se o pedido não existir, insere o novo pedido
          console.log("Inserting new order");

          await new Promise((resolve, reject) => {
            this.db.query(
              `INSERT INTO pedidos_func (id_func, id_produto_func, quantidade) VALUES (?, ?, ?)`,
              [data.id_func, data.id_produto, data.quantidade],
              (error) => {
                if (error) {
                  console.error("Error inserting order:", error);
                  return reject(error);
                }
                resolve();
              }
            );
          });
        }

        return resolve({ success: "Pedido adicionado com sucesso!", code: 200 });
      } catch (error) {
        console.error('Erro ao registrar pedido:', error);
        resolve({ error: "STATIC_ERRO_VALIDACAO_SQL", code: 203 });
      }
    });
  }


    /**
     * Deleta o usuario cadastrado
     * @param {*} id ID do usuario a ser excluído
     * @returns mensagem referente ao resultado obtido
     */
    delete(id) {
        return new Promise((resolve, reject) => {
            this.db.query(
                "DELETE FROM user WHERE id = ?",
                [id],
                (error, response) => {
                    if (error) {
                        return resolve({
                            error: "Este usuarios possui vinculos!",
                            code: 203,
                        });
                    } else {
                        return resolve(true);
                    }
                }
            );
        });
    }

    /**
     * Atualiza os dados de um usuario
     * @param {*} data dados atualizados do usuario
     * @returns mensagem referente ao resultado obtido
     */
    update(data) {
        return new Promise(async (resolve, reject) => {
            this.db.query(
                "UPDATE usuarios SET nome=?, email=?, perfil=? WHERE id=?",
                [data.nome, data.email, data.perfil, data.id],
                async (error, response) => {
                    if (error) return reject(new Error(error));
                    else return resolve({ success: "Usuario atualizado com sucesso!" });
                }
            );
        });
    }

  /**
* Atualiza a quantidade do pedido de um funcionario
* @param {*} data dados atualizados da pedido
* @returns mensagem referente ao resultado obtido 
*/
  updateQuantidade(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        "UPDATE pedidos SET quantidade = ? WHERE id_func = ? AND id_produto_func = ? AND concluido = false",
        [
          data.quantidade,
          data.id_func,
          data.id_produto,
        ],
        (error, response) => {
          if (error) return reject(new Error(error));
          else return resolve({ success: "Quantidade atualizada com sucesso!" });
        }
      );
    });
  }

  
  /**
   * Deleta uma um produto de um pedido de um funcionario
   * @param {*} id Id da unidade a ser excluída
   * @returns mensagem referente ao resultado obtido
   */
  deleteProdutoDoPedido(id_produto, id_func) {
    return new Promise((resolve, reject) => {

      this.db.query('DELETE FROM pedidos_func WHERE id_produto_func = ? AND id_func = ? AND concluido = false', [id_produto, id_func],
        (error, response) => {
          if (error)  return resolve({ error: "Este pedido possui vinculos!", code: 203 });
          else return resolve(true);
        });
    });
  }


  listTabelaDePrecoFunc(id_func) {
    return new Promise(async(resolve, reject) => {
      try {

        const user = await new Promise((resolve, reject) => {
          this.db.query(
              `SELECT * from funcionarios WHERE id = ?`,[id_func],
              (error, funcionario) => {
                  if (error) {
                      console.error("Error: ", error);
                      return reject(error);
                  }
                  resolve(funcionario[0]);
              });
      });

      let categoriaFilter = '';

      // if (user.role === 'FUNC') categoriaFilter = 'p.categoria IN (1,2)';
      // else if (user.role === 'USER') categoriaFilter = 'p.categoria = 1';

      const produtos = await new Promise((resolve, reject) => {
        this.db.query(
          `SELECT p.id, p.linha, p.categoria, p.ordem, p.preco5, p.descricao_unidade AS descricao, p.nome_unidade AS nome
            FROM produtos p
            WHERE
              p.visivel = true AND
              p.excluido = false
            ORDER BY p.ordem`,
          [],
          (error, produtos) => {
            if (error) {
              console.log(error);
              return reject(new Error(error));
            }
            return resolve(produtos);
          }
        );
      })
        return resolve(produtos)
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }


  /**
   * Conclui um pedido de um fucionario 
   * @param {*} data dados de um pedido a se concluido
   * @returns mensagem referente ao resultado obtido
   */
  concluirPedido(packet) {
    return new Promise(async (resolve, reject) => {
      try {
        const dateAtual = new Date();
        const horaFormatada = dateAtual.toLocaleTimeString('pt-BR', { hour: '2-digit', minute: '2-digit' });
        const dataFormatada = dateAtual.toISOString().split('T')[0];
        let vendedor;


        const ultra = await new Promise((resolve, reject) => {
          this.db.query(
              `SELECT * FROM ultra`,[],
              (error, ultra) => {
                  if (error) {
                      console.error("Error: ", error);
                      return reject(error);
                  }
                  resolve(ultra[0]);
              }
          );
      });


    
        const pedido = await new Promise((resolve, reject) => {
          this.db.query(
              `SELECT pedidos_func.id_func, pedidos_func.id_produto_func, pedidos_func.quantidade FROM pedidos_func WHERE concluido = false AND id_func = ?`, [packet.id_user],
              (error, pedido) => {
                  if (error) {
                      console.error("Error fetching orders: ", error);
                      return reject(error);
                  }
                  resolve(pedido);
              }
          );
      });
      
      // const descontos = await Promise.all(
      //     pedido.map(p =>
      //         new Promise((resolve, reject) => {
      //             this.db.query(
      //                 `SELECT * FROM descontos WHERE id_produto = ? AND id_func = ?`, [p.id_produto, packet.id_func],
      //                 (error, desconto) => {
      //                     if (error) {
      //                         console.error(`Error fetching discount for product ${p.id_produto}: `, error);
      //                         return reject(error);
      
                              
      //                     }
      //                     resolve(desconto[0] || { percentual: 0 }); // Retorna 0% desconto se não houver registro.
      //                 }
      //             );
      //         })
      //     )
      // );
      
      if (ultra.ultra == 1) { 
      const produtos = await Promise.all(
          pedido.map(p =>
              new Promise((resolve, reject) => {
                  this.db.query(
                      `SELECT * FROM produtos_ultra WHERE codProduto = ?`, [p.id_produto],
                      (error, result) => {
                          if (error) {
                              console.error(`Error fetching product ${p.id_produto}: `, error);
                              return reject(error);
                          }
                          resolve(result[0]);
                      }
                  );
              })
          )
      );
      
      
      let ultimoId = await getLastId()+1
      
      const itens = produtos.map((produto, index) => {
        const precoOriginal = produto.preco;
        const qtdPedido = pedido[index].quantidade;
        // const descontoPercentual = descontos[index].desconto || 0; 
      
        const valorDescontoUnitario = precoOriginal;
      
        // Preço líquido por unidade após o desconto
        const precoLiquidoUnitario = precoOriginal - valorDescontoUnitario;
      
        // Cálculo do valor total (para a quantidade do pedido)
        const valorTotalOriginal = precoOriginal * qtdPedido;
        const valorDesconto = valorDescontoUnitario * qtdPedido;
        const valorLiquido = precoLiquidoUnitario * qtdPedido;
      
        console.log("VALOR DESCONTO: ", valorDesconto);
        console.log("VALOR LIQUIDO: ", valorLiquido);
      
        console.log("VALOR DESCONTO UNI ", valorDescontoUnitario);
        
        
        return {
            codGrupoNivel1: produto.codGrupoNivel1,
            codGrupoNivel2: produto.codGrupoNivel2,
            codGrupoNivel3: produto.codGrupoNivel3,
            codPedido: ultimoId,
            codProduto: produto.codProduto,
            codProdutoClas: produto.codProdutoClass,
            codTributacaoUF: 0,
            codTributacaoUFDest: 0,
            codUnidadeMedida: 0,
            descUnidadeMedida: "",
            grupoFiscal: produto.grupoFiscal,
            item: index + 1,
            margemST: 0.0,
            perIPI: 0.0,
            percBaseICMSUF: 0.0,
            percBaseICMSUFDest: 0.0,
            percICMSUF: 0.0,
            percICMSUFDest: 0.0,
            pesoEmbalagem: produto.pesoEmbalagem,
            pesoRepesagem: "0,0000",
            preco: precoLiquidoUnitario,
            qtdAtual: 0.0,
            qtdPedido: qtdPedido,
            qtdUnMed: produto.pesoProduto,
            quantidade: qtdPedido,
            unidadeMedida: produto.undMedida,
            valorBaseICMSUF: 0.0,
            valorBaseICMSUFDest: 0.0,
            valorBaseIPI: 0.0,
            valorBaseST: 0.0,
            valorBruto: valorLiquido.toFixed(2),
            valorDesconto: valorDescontoUnitario,
            valorICMSUF: 0.0,
            valorICMSUFDest: 0.0,
            valorIPI: 0.0,
            valorInformado: precoLiquidoUnitario,
            valorLiquido: valorLiquido.toFixed(2),
            valorRepesagem: "0.00",
            valorST: 0.0,
            valorTotal: valorLiquido.toFixed(2),
            vlrAtual: 0.0,
            vlrTotalAtual: 0.0
        };
      });
      
      // Calcula o valor líquido total para o envelope SOAP.
      const valorLiquidoTotal = itens.reduce((acc, item) => acc + parseFloat(item.valorLiquido), 0);
      
      const data = new Date().toISOString().split('T')[0];
      const dataAtual = new Date();
      
      dataAtual.setDate(dataAtual.getDate() + 2);
      
      const dataEntrega = dataAtual.toISOString().split('T')[0];
      
      const soapEnvelope = `<?xml version="1.0" encoding="utf-8"?>
      <soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
      xmlns:xsd="http://www.w3.org/2001/XMLSchema"
      xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
      <soap12:Body>
      <SaveAllOrders xmlns="http://www.ultrasistemas.com.br/">
      <strPedidos>
      [{
      "codCondRec": 4,
      "codFilial": 1,
      "codForma": 2,
      "codOperacao": 100,
      "codTabelaPreco": 1,
      "codVendedor": 10156,
      "cpfCnpj": "372.236.674-78",
      "data": "${data}",
      "dataEntrega": "${dataEntrega}",
      "emailCliente": "",
      "id":  ${ultimoId},
      "itens": ${JSON.stringify(itens)},
      "md5": "${crypto.randomBytes(9).toString('hex').slice(0, 12)}",
      "newCodigoFV": 0,
      "observacao": "PEDIDO TESTE NAO FATURAR",
      "parceiro": 10204,
      "codigo": ${ultimoId},
      "pesoBruto": 0.0,
      "pesoLiquido": 0.0,
      "qtdeVolumes": 0,
      "valorBaseST": 0.0,
      "valorDesconto": "${itens.reduce((acc, item) => acc + parseFloat(item.valorDesconto), 0).toFixed(2)}",
      "valorIPI": 0.0,
      "valorBruto": "0.00",
      "valorST": 0.0,
      "valorLiquido": "0.00"
      }]
      </strPedidos>
      </SaveAllOrders>
      </soap12:Body>
      </soap12:Envelope>`;
      
      
      await saveLastId(ultimoId);
      
      
          try {
            const response = await axios.post(this.wsdlUrl, soapEnvelope, {
              headers: {
                'Content-Type': 'application/soap+xml; charset=utf-8',
                'SOAPAction': this.soapAction, 
              },
            });
        
            console.log('SOAP Response: ', response);
            
            
          } catch (error) {
            console.error("CATCH ERROR: ", error);
            if (error.response) console.error("Status:", error.response.status);
          }
        }


        const funcionario = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT * FROM funcionarios WHERE id = ?`, [packet.id_func],
            (error, results) => {
              if (error) {
                console.error("Erro ao buscar funcionario: ", error);
                return reject(error);
              }
              resolve(results[0]);
            }
          );
        });
        
        funcionario.id_vendedor !== null ?
           vendedor = await new Promise((resolve, reject) => {
            this.db.query(
              `SELECT * FROM vendedores WHERE id = ?`, [funcionario.id_vendedor],
              (error, results) => {
                if (error) {
                  console.error("Erro ao buscar usuário: ", error);
                  return reject(error);
                }
                resolve(results[0]);
              }
            );
          })
          : ``
  
        
        const produtosPedidos = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT produtos_func.nome, produtos_func.id AS id_produto, pedidos_func.quantidade, pedidos_func.aroma, produtos_func.nome_unidade, produtos_func.descricao_unidade
             FROM produtos_func
             INNER JOIN pedidos_func ON pedidos_func.id_produto_func = produtos_func.id
             WHERE pedidos_func.id_func = ? AND pedidos_func.concluido = false`,
            [packet.id_func],
            (error, results) => {
              if (error) {
                console.error("Erro ao buscar produtos do pedido: ", error);
                return reject(error);
              }
              resolve(results);
            }
          );
        });
        
        
        if (produtosPedidos.length == 0) return resolve({ error: `Adicione itens no carrinho`, code: 203 })

        
  
        // Atualiza pedido no banco de dados
        await new Promise((resolve, reject) => {
          this.db.query(
            `UPDATE pedidos_func SET concluido = ?, data = ?, hora = ?, aroma = ?, observacao = ?
             WHERE id_func = ? AND concluido = false`,
            [packet.concluido, dateAtual, horaFormatada, packet.aroma, packet.observacao, packet.id_func],
            (error) => {
              if (error) {
                console.error("Erro ao atualizar pedido: ", error);
                return reject(error);
              }
              resolve();
            }
          );
        });
        console.log(produtosPedidos);
        

      const mensagemCliente = `*Aliança Pedidos - Pedido Realizado!*\n
${produtosPedidos.map((p, index) => `*Produto ${index+1}*
- ${p.nome_unidade}
- ${p.descricao_unidade.toLowerCase()}
- ${p.quantidade} Unidades\n`).join('\n')}
*Observações do Pedido:*
${packet.observacao || 'Nenhuma observação adicionada'}\n
\n
*Para mais pedidos acesse:*\nhttps://funcionarios.aliancapedidos.com.br`


  let sellerMessage

vendedor?.contrato === "RH" ?
        sellerMessage = `${funcionario.nome || 'Não disponível'}
${funcionario.documento}\n
${produtosPedidos.map(p => `${p.id_produto} = ${p.quantidade} Unidades`).join('\n')}
${packet.observacao ? `\n${packet.observacao}` : '\nNenhuma observação adicionada'}`
:
sellerMessage = `${funcionario.nome || 'Não disponível'}
${funcionario.documento}\n
${produtosPedidos.map(p => `${p.id_produto} = ${p.quantidade} Caixas`).join('\n')}
${packet.observacao ? `\n${packet.observacao}` : '\nNenhuma observação adicionada'}`


          const fotosAWS = ["pedidos1", "pedidos3", "pedidos4"]
          const indiceAleatorio = Math.floor(Math.random() * fotosAWS.length);

          console.log(fotosAWS[indiceAleatorio]);

        await sendMedia(mensagemCliente,  "+55"+funcionario.celular, fotosAWS[indiceAleatorio])

      funcionario.id_vendedor ?
          await sendMessage(sellerMessage, "+55"+vendedor.celular) 
          : 
          ""

          
          // await sendMessage(sellerMessage, "+5519995201285")

        return resolve({ success: "Pedido concluído!", code: 200 });
      } catch (error) {
        console.error("Erro ao concluir pedido: ", error);
        return reject(new Error("Erro ao concluir pedido."));
      }
    });
  }
}

module.exports = funcionarioRepository;
