const {
  encrypt,
  generateHash,
  checkHash,
} = require("../../config/auth/cryptography");
const { genereteToken } = require("../../config/auth/jwt");
const { decryptDonor } = require("../../util/cryptDonor");
const db = require("../database/db");
const nodemailer = require("nodemailer");
const crypto = require("crypto");
const fs = require('fs');
const path = require("path");
const bcrypt = require('bcrypt');
const { validarCPF } = require("../../util/validarCPF")
const { validarCNPJ } = require("../../util/validarCNPJ")
const { trocarAno } = require("../../util/dateFormat");
const multer = require("multer");
const sharp = require("sharp")
const { getSignedUrl } = require("@aws-sdk/s3-request-presigner");
const { S3Client, PutObjectCommand, GetObjectCommand } = require('@aws-sdk/client-s3');
const { validarNumeroCelularLib } = require("../../util/validacaoCelularLib");
require("dotenv").config({ path: __dirname + "/./../../../.env" });



class vendedoresRepository {
  constructor() {
    this.db = db;
    this.s3Client = new S3Client({
      region: process.env.S3_REGION,
      credentials: {
        accessKeyId: process.env.AWS_ACCESS_KEY,
        secretAccessKey: process.env.AWS_SECRET_KEY,
      },
    });

  }

  /**
   * Registra os dados de um usuario
   * @param {*} data dados do usuario
   * @returns mensagem referente ao resultado obtido
   */
  register(data) {
    return new Promise(async (resolve, reject) => {

      let resultado;
      if (data.documento.length == 11) resultado = validarCPF(data.documento)
      if (data.documento.length == 14) resultado = validarCNPJ(data.documento)


      if (!resultado) return resolve({ error: `CNPJ ou CPF inexistente!`, code: 203 })



      this.db.query(
        `INSERT INTO vendedores (nome, celular, email, documento, contrato, endereco) VALUES(?,?,?,?,?,?)`,
        [data.nome, data.celular, data.email, data.documento, data.contrato, data.endereco],
        async (error, response) => {
          if (error) {
            console.error(error);
            resolve({ error: "Erro ao registrar vendedor", code: 203 });
          }
          return resolve({ success: "Vendedor cadastrado com sucesso!", code: 200 });
        }
      );
    });
  }

  
  loginUser(data, documento) {
    return new Promise(async (resolve, reject) => {



      String(documento).trim(); 

      const user = await new Promise(async (resolve, reject) => {
        this.db.query(
          `SELECT * FROM user WHERE documento like ?`,
          [documento],
          async (error, response) => {
            if (error) {
              console.error("Database query error: ", error);
              return reject(new Error("Database query error"));
            }
            resolve(response[0])
          })
      });


      if (!user || user.length === 0) {
        return resolve({ error: `Usuário não encontrado`, code: 203 });
      }


      const auth = crypto.randomBytes(128).toString("HEX");

      this.db.query(
        "UPDATE user SET auth=? WHERE documento=?",
        [auth, documento],
        async (updateError, updateResult) => {
          if (updateError) {
            console.error("Database update error:", updateError);
            return reject(new Error("Database update error"));
          }

          if (updateResult.affectedRows === 0) {
            return reject(new Error("Nenhuma linha foi atualizada. Verifique se o documento existe."));
          }

          return resolve({ id: user.id });
        }
      );


    });
  }


  loginVendedor(data) {
    return new Promise(async (resolve, reject) => {


      this.db.query(
        `SELECT * FROM user WHERE documento = ?  `,
        [data.documento],
        async (error, response) => {
          if (error) {
            console.error("Database query error: ", error);
            return reject(new Error("Database query error"));
          }

          if (response.length === 0) return resolve({ error: `Vendedor não encontrado`, code: 203 });

          const userDB = response[0];

          const isMatch = await bcrypt.compare(data.senha, userDB.senha);

          if (!isMatch) return resolve({ error: "Senha incorreta", code: 401 });


          const auth = crypto.randomBytes(128).toString("HEX");
          userDB.auth = auth;

          this.db.query(
            "UPDATE user SET auth=? WHERE documento=?",
            [auth, data.documento],
            async (updateError) => {
              if (updateError) {
                console.error("Database update error:", updateError);
                return reject(new Error("Database update error"));
              }



              try {
                const token = await genereteToken(userDB);

                return resolve({ id: userDB.id });
              } catch (tokenError) {
                console.error("Token generation error: ", tokenError);
                return reject(new Error("Token generation error"));
              }
            }
          );
        }
      );
    });
  }



  /**
   * Realiza o login de um vendedor
   * Caso o vendedor nao tenha uma senha definida, sera cadastrado a senha que ele enviara nessa rota pela primera vez
   * @param {*} data dados do vendedor
   * @returns mensagem referente ao resultado obtido
   */
  login(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        "SELECT * FROM vendedores WHERE email=?",
        [data.email],
        async (error, response) => {
          if (error) {
            console.error("Database query error: ", error);
            return reject(new Error("Database query error"));
          }

          if (response.length === 0) return resolve({ error: "Vendedor não encontrado", code: 204 });

          const userDB = response[0];

          // Se a senha não está definida no bancod
          if (!userDB.senha) {
            try {
              // Criptografa a nova senha
              const hashedPassword = await bcrypt.hash(data.senha, 10);

              // Atualiza a senha no banco de dados
              this.db.query(
                "UPDATE vendedores SET senha=? WHERE email=?",
                [hashedPassword, data.email],
                (updateError) => {
                  if (updateError) {
                    console.error("Error updating password: ", updateError);
                    return reject(new Error("Error updating password"));
                  }
                }
              );
            } catch (hashError) {
              console.error("Error hashing password: ", hashError);
              return reject(new Error("Error hashing password"));
            }
          } else {
            // Verifica se a senha fornecida corresponde à senha armazenada
            const isMatch = await bcrypt.compare(data.senha, userDB.senha);
            if (!isMatch) return resolve({ error: "Senha incorreta", code: 401 });
          }

          const auth = crypto.randomBytes(128).toString("HEX");
          userDB.auth = auth;

          // Atualiza o banco de dados com o novo auth
          this.db.query(
            "UPDATE vendedores SET auth=? WHERE email=?",
            [auth, data.email],
            async (updateError) => {
              if (updateError) {
                console.error("Database update error:", updateError);
                return reject(new Error("Database update error"));
              }

              this.db.query(
                `INSERT INTO logs (id_user, data, acao) VALUES (?,?,?)`,
                [userDB.id, new Date(), "LOGIN VENDEDOR"],
                (logError) => {
                  if (logError) {
                    console.error("Error inserting logs: ", logError);
                    return reject(logError);
                  }
                }
              );

              try {
                const token = await genereteToken(userDB);

                return resolve({
                  token: token,
                  user: {
                    id: userDB.id,
                    nome: userDB.nome,
                    documento: userDB.documento,
                    email: userDB.email,
                    celular: userDB.celular,
                    contrato: userDB.contrato,
                    endereco: userDB.endereco,
                    auth: auth,
                  },
                });
              } catch (tokenError) {
                console.error("Token generation error: ", tokenError);
                return reject(new Error("Token generation error"));
              }
            }
          );
        }
      );
    });
  }
  /**
   * Envia um email de um vendedor para um cliente
   * @param {*} data dados do email a ser enviado
   * @returns mensagem com o resultado obtido
   */
  // emailToUser(data) {
  //   return new Promise(async (resolve, reject) => {

  //     const vendedor = await new Promise((resolve, reject) => {
  //       this.db.query(
  //         `SELECT * FROM vendedores WHERE id = ?`,[data.id],
  //         (error, results) => {
  //           if (error) {
  //             console.error("Error fetching existing saller: ", error);
  //             return reject(error);
  //           }
  //           resolve(results);
  //         }
  //       );
  //     });



  //     let transporter = nodemailer.createTransport({
  //       host: 'brasapedidos.com.br',
  //       port: 465,
  //       secure: true,
  //       auth: {
  //         user: process.env.EMAIL_SENDER,
  //         pass: process.env.EMAIL_PASSWORD,
  //       },
  //     });

  //     let mailOptions = {
  //       from: `suporte@brasapedidos.com.br`,
  //       to: data.email,
  //       subject: data.subject,
  //       // text: `${newPassword}`,
  //       html: `vendedor-cliente`
  //     };

  //     transporter.sendMail(mailOptions, (error, info) => {
  //       if (error) {
  //         return console.log(error);
  //       }
  //       console.log('Email sent: %s', info.messageId);
  //       return resolve({ success: "Email enviado com sucesso!", code: 200 });
  //     });


  //   });
  // }


emailToUser(data) {
  return new Promise(async (resolve, reject) => {
    try {
      // Obter dados do vendedor
      const vendedor = await new Promise((resolve, reject) => {
        this.db.query(
          `SELECT * FROM vendedores WHERE id = ?`, [data.id],
          (error, results) => {
            if (error) {
              console.error("Error fetching existing seller: ", error);
              return reject(error);
            }
            resolve(results[0]); // Assume-se que há apenas um resultado
          }
        );
      });

      // Ler template HTML
      const templatePath = path.join(__dirname, '../../view/vendedor-cliente.hbs');
      const templateHtml = fs.readFileSync(templatePath, 'utf-8');

      // Substituir variáveis no template
      const emailHtml = templateHtml
        // .replace('{{nomeUsuario}}', data.nome)
        .replace('{{mensagem}}', data.mensagem)
        // .replace('{{nomeVendedor}}', vendedor.nome)


      // Configurar o transporte do email
      let transporter = nodemailer.createTransport({
        host: 'brasapedidos.com.br',
        port: 465,
        secure: true,
        auth: {
          user: process.env.EMAIL_SENDER,
          pass: process.env.EMAIL_PASSWORD,
        },
      });

      let mailOptions = {
        from: `suporte@brasapedidos.com.br`,
        to: data.email,
        subject: data.subject,
        html: emailHtml,
      };

      transporter.sendMail(mailOptions, (error, info) => {
        if (error) {
          console.error("Error sending email: ", error);
          return reject(error);
        }
        console.log('Email sent: %s', info.messageId);
        resolve({ success: "Email enviado com sucesso!", code: 200 });
      });
    } catch (error) {
      console.error("Error in emailToUser: ", error);
      reject(error);
    }
  });
}

updateProduto(data, id, id_vendedor) {
  return new Promise(async (resolve, reject) => {



    const vendedor = await new Promise((resolve, reject) => {
      this.db.query(
        `SELECT * FROM vendedores WHERE id = ?`,
        [id_vendedor],
        (error, results) => {
          if (error) {
            console.error("Error fetching existing order:", error);
            return reject(error);
          }
          resolve(results[0]);
        }
      );
    });

    if (vendedor.contrato !== "RH") {
        this.db.query(
          "UPDATE produtos SET nome=?, descricao=?, image=?, preco1=?, preco2=?, preco3=?, preco4=?, preco5=?, categoria=?, linha=?, visivel=?, excluido=?, exclusivo=? WHERE id = ?",
          [data.nome, data.descricao, data.image, data.preco1, data.preco2, data.preco3, data.preco4, data.preco5, data.categoria, data.linha, data.visivel, data.excluido, data.exclusivo, id],
          (error, response) => {
            if (error) {
              console.error(error);
              return reject(new Error(error)); 
            }
            return resolve({ success: "Produto atualizado com sucesso!", code: 200 });
          }
        );
      }
      else {

    this.db.query(
      "UPDATE produtos_func SET nome=?, descricao=?, image=?, preco1=?, preco2=?, preco3=?, preco4=?, preco5=?, categoria=?, linha=?, visivel=?, excluido=?, exclusivo=? WHERE id = ?",
      [data.nome, data.descricao, data.image, data.preco1, data.preco2, data.preco3, data.preco4, data.preco5,  data.categoria, data.linha, data.visivel, data.excluido, data.exclusivo, id],
      (error, response) => {
        if (error) {
          console.error(error);
          return reject(new Error(error)); 
        }
         return resolve({ success: "Produto atualizado com sucesso!", code: 200 });
      }
    );
  }
  });
}



  recover(data) {
    return new Promise((resolve, reject) => {
      // Verificar se o email existe no banco de dados
      db.query("SELECT email FROM vendedores WHERE email = ?", [data.email], async (err, result) => {
        if (err) return reject("Erro no banco de dados");

        if (result.length === 0) return resolve({ error: "Email não encontrado!", code: 204 });

        let newPassword = Math.floor(100000 + Math.random() * 900000);
        newPassword += ""

        const hashedPassword = await bcrypt.hash(newPassword, 10);

        // Atualizar a senha no banco de dados
        this.db.query("UPDATE vendedores SET senha = ? WHERE email = ?", [hashedPassword, data.email], (err) => {
          if (err) {
            console.log(err);

            return reject("Erro ao atualizar a senha no banco de dados");
          }

          // Configuração do transporte de email
          let transporter = nodemailer.createTransport({
            host: 'brasapedidos.com.br',
            port: 465,
            secure: true,
            auth: {
              user: process.env.EMAIL_SENDER,
              pass: process.env.EMAIL_PASSWORD,
            },
          });

          let mailOptions = {
            from: 'suporte@brasapedidos.com.br',
            to: data.email,
            subject: 'Recuperação de senha',
            // text: `${newPassword}`,
            html: `
            <!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Password Recovery</title>
    <style>
        body {
            font-family: Arial, sans-serif;
            background-color: #f4f4f4;
            margin: 0;
            padding: 0;
        }
        .container {
            max-width: 500px; /* Define a largura máxima */
            width: 100%;
            margin: 40px auto;
            background-image: linear-gradient(180deg, #B91C1C, #DC2626);
            color: white;
            padding: 20px;
            border-radius: 8px;
            box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1);
        }
        h1 {
            font-size: 24px;
            color: #f3f4f6;
            text-align: center;
            margin-bottom: 20px;
        }
        p {
            font-size: 16px;
            color: #f3f4f6;
            line-height: 1.5;
            margin: 0 0 20px;
        }
        .new-password {
            display: block;
            font-size: 20px;
            font-weight: 600;
            color: #ffffff;
            border: 2px solid rgba(255, 255, 255, 0.7);
            background-color: rgba(255, 255, 255, 0.1);
            padding: 15px;
            border-radius: 4px;
            text-align: center;
            margin: 20px 0;
        }
        .footer {
            text-align: center;
            font-size: 14px;
            color: #f3f4f6;
            margin-top: 20px;
        }
    </style>
</head>
<body>
    <div class="container">
        <h1>Senha Alterada</h1>
        <p>Sua senha foi alterada com sucesso!</p>
        <p>Sua nova senha é:</p>
        
        <div class="new-password">
            ${newPassword}
        </div>
        
        <p>Se você não solicitou essa mudança, ignore este e-mail.</p>
        
        <div class="footer">
            &copy; 2025 Aliança
        </div>
    </div>
</body>
</html>
            `
          };

          transporter.sendMail(mailOptions, (error, info) => {
            if (error) {
              return console.log(error);
            }
            console.log('Email sent: %s', info.messageId);
          });

          return resolve({ success: "Nova senha enviada para o email!" });

        });
      });
    });
  }



  /**
   * Lista os dados de um vendedor
   * @returns lista com todos os dados de um usuario
   */
  list(id) {
    return new Promise(async (resolve, reject) => {
      var query = id ? `WHERE id = ?` : "";


      try {
        this.db.query(
          `SELECT * FROM vendedores ${query}`,
          [id ? id : ""],

          (error, vendedores) => {
            if (error) return reject(new Error(error));
            return resolve(vendedores);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
   * Deleta o vendedor cadastrado
   * @param {*} id ID do vendedor a ser excluído
   * @returns mensagem referente ao resultado obtido
   */
  delete(id) {
    return new Promise((resolve, reject) => {
      this.db.query(
        "CALL deletarVendedor(?)",
        [id],
        (error, response) => {
          if (error) {
            return resolve({
              error: "Este vendedor possui vinculos!",
              code: 203,
            });
          } else {
            return resolve(true);
          }
        }
      );
    });
  }

  /**
   * Atualiza os dados de um usuario
   * @param {*} data dados atualizados do usuario
   * @returns mensagem referente ao resultado obtido
   */
  update(data) {
    return new Promise(async (resolve, reject) => {

      let resultado;
      if (data.documento.length == 11) resultado = validarCPF(data.documento)
      if (data.documento.length == 14) resultado = validarCNPJ(data.documento)


      if (!resultado) return resolve({ error: `CNPJ ou CPF inexistente!`, code: 203 })

      this.db.query(
        "UPDATE vendedores SET nome=?, email=?, celular=?, documento=?, contrato=?, endereco=? WHERE id=?",
        [data.nome, data.email, data.celular, data.documento, data.contrato, data.endereco, data.id],
        async (error, response) => {
          if (error) return reject(new Error(error));
          else return resolve({ success: "Usuario atualizado com sucesso!" });
        }
      );
    });
  }

  /**
   * Lista quantidade minima de produto que um user pode comprar
   * @param {*} id id do usuario
   * @returns quatidade minima de produto de um usuario
   */
  listQtdeMinima(id) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        `SELECT user.id, user.qtde_minima_suino, user.qtde_minima_bovino FROM user WHERE id = ?`,
        [id],
        async (error, user) => {
          if (error) return reject(new Error(error));
          return resolve(user);
        }
      );
    });
  }

  /**
   * Lista todos os ultimos pedidos dos clientes vinculado ao id_vendedor fornecido
   * @param {int} id_vendedor id do vendedor
   * @returns lista com todos os pedidos dos clientes do vendedor
   */
  listUltimosPedidos(id_vendedor) {
    return new Promise(async (resolve, reject) => {


      this.db.query(
        `SELECT pedidos.id, user.id AS id_user, user.razao_social AS cliente, pedidos.quantidade, DATE_FORMAT(pedidos.data, "%d/%m/%Y") AS data, pedidos.hora
            FROM user, pedidos, vendedores
            WHERE
            user.id_vendedor = vendedores.id AND
            pedidos.id_user = user.id AND
            user.id_vendedor = ?
            ORDER BY pedidos.data DESC, pedidos.hora DESC`,
        [id_vendedor],
        async (error, response) => {
          if (error) return reject(new Error(error));
          return resolve(response);
        }
      );
    });
  }

  /**
   * Lista todos os clientes que o vendedor possui vinculo
   * @param {int} id_vendedor id do vendedor
   * @returns Lista com todos os clientes do vendedor 
   */
  listClientesDoVendedor(id_vendedor) {
    return new Promise(async (resolve, reject) => {

      if (!id_vendedor) return resolve({ error: `id do vendedor obrigatorio!`, code: 203 })



      this.db.query(
        `SELECT user.id, user.razao_social, user.documento, user.celular, user.email, user.role, user.status, user.tipo, user.endereco, user.qtde_minima_bovino, user.qtde_minima_suino,
            user.id_vendedor, user.anotacoes, DATE_FORMAT(user.data_criado, "%d/%m/%Y") AS data_criado, user.dias_pedido, user.aroma, user.meta_mensal
            FROM user, vendedores
            WHERE
            user.id_vendedor = vendedores.id AND
            user.id_vendedor = ?`,
        [id_vendedor],
        async (error, response) => {
          if (error) return reject(new Error(error));

          const usuariosTransformados = response.map(usuario => ({
            ...usuario,
            dias_pedido: usuario.dias_pedido ? usuario.dias_pedido.match(/Segunda|Terça|Quarta|Quinta|Sexta/g) || [] : []
          }));
          return resolve(usuariosTransformados);
        }
      );
    });
  }
  /**
   * Lista quantidade total vendida de produtos de clientes vinculado a o vendedor fornecido 
   * @param {Int} id_vendedor  ID do vendedor 
   * @returns Quantidade total vendida de produtos
   */
  listQtdeVendidaPorVendedor(id_vendedor, data_inicio1, data_fim1, data_inicio2, data_fim2) {
    return new Promise(async (resolve, reject) => {

      if (!id_vendedor) return resolve({ error: `id do vendedor obrigatorio!`, code: 203 })

        let params = [
          data_inicio1, data_fim1,
          data_inicio2, data_fim2,
          data_inicio1, data_fim1,
          data_inicio2, data_fim2,
          data_inicio2, data_fim1,
          id_vendedor,                
      ];

      let query = `SELECT 
        produtos.id AS id_produto, 
        produtos.nome, 
        COALESCE(SUM(CASE 
                        WHEN pedidos.data BETWEEN ? AND ?
                        THEN pedidos.quantidade 
                    END), 0) AS quantidade_vendida_1,
        COALESCE(SUM(CASE 
                        WHEN pedidos.data BETWEEN ? AND ? 
                        THEN pedidos.quantidade 
                    END), 0) AS quantidade_vendida_2,
        COALESCE(SUM(CASE 
                        WHEN pedidos.data BETWEEN ? AND ?
                        THEN pedidos.quantidade 
                    END), 0) - 
        COALESCE(SUM(CASE 
                        WHEN pedidos.data BETWEEN ? AND ?
                        THEN pedidos.quantidade 
                    END), 0) AS diferenca

            FROM pedidos
            JOIN user ON pedidos.id_user = user.id
            JOIN produtos ON pedidos.id_produto = produtos.id
            JOIN vendedores ON user.id_vendedor = vendedores.id

            WHERE pedidos.concluido = TRUE
            AND pedidos.data BETWEEN ? AND ?
            AND user.id_vendedor = ?

            GROUP BY produtos.id, produtos.nome 

            ORDER BY quantidade_vendida_1 DESC`

      this.db.query(
        query,

            
        params,
        async (error, response) => {
          if (error) return reject(new Error(error));
          return resolve(response);
        }
      );
    });
  }

  /**
 * Lista quantidade total de pedidos concluidos por tipos de contratos de vendedor e total de vendas dentro das duas datas fornecidas
 * @param {*} dataInicio Para inicio da filtro
 * @param {*} dataFim Para fim do filtro
 * @returns Todos dias com seus pedidos concluidos se o resultado for 0 significa que nao houve pedido fechado naquele dia
 */
  listQtdeTotalVendedoresContrato(dataInicio, dataFim) {
    return new Promise(async (resolve, reject) => {

      if (!dataInicio || !dataFim) return resolve({ error: `Data não atribuida!`, code: 203 })

      dataInicio = trocarAno(dataInicio)
      dataFim = trocarAno(dataFim)

      this.db.query(
        `WITH recursive AllDates AS (
            SELECT DATE(?) AS data -- Início do intervalo
            UNION ALL
            SELECT data + INTERVAL 1 DAY
            FROM AllDates
            WHERE data + INTERVAL 1 DAY <= DATE(?)
          )
          SELECT
              DATE_FORMAT(AllDates.data, "%d/%m") AS data,
              COALESCE(SUM(CASE WHEN vendedores.contrato = 'CLT' THEN pedidos.quantidade ELSE 0 END), 0) AS total_clt,
              COALESCE(SUM(CASE WHEN vendedores.contrato = 'PJ' THEN pedidos.quantidade ELSE 0 END), 0) AS total_pj,
              COALESCE(SUM(CASE WHEN user.id_vendedor IS NULL THEN pedidos.quantidade ELSE 0 END), 0) AS total_sem_vinculo,
              COALESCE(SUM(pedidos.quantidade), 0) AS total_vendas
          FROM 
              AllDates
          LEFT JOIN 
              pedidos ON DATE(pedidos.data) = AllDates.data
          LEFT JOIN 
              user ON pedidos.id_user = user.id
          LEFT JOIN 
              vendedores ON user.id_vendedor = vendedores.id
          WHERE 
              pedidos.concluido = 1 OR pedidos.concluido IS NULL
          GROUP BY 
              AllDates.data
          ORDER BY 
              AllDates.data
            Limit 366`,
        [dataInicio, dataFim],
        async (error, user) => {
          if (error) return reject(new Error(error));
          return resolve(user);
        }
      );
    });
  }

  /**
   * Lista os 5 produtos mais pedidos de um usuario
   * @param {integer} id_user 
   * @returns lista com os 5 produtos mais pedidos de um usuario
   */
  listTopProdutos(id_user, periodo) {
    return new Promise(async (resolve, reject) => {

      let query = ``;
      switch (periodo) {
        case 'semanal':
          query = `YEARWEEK(pedidos.data, 1) = YEARWEEK(CURDATE(), 1) AND`
          console.log('Ação para semanal');
          break;
        case 'mensal':
          query = `YEAR(pedidos.data) = YEAR(CURDATE()) AND MONTH(pedidos.data) = MONTH(CURDATE()) AND`
          console.log('Ação para mensal');
          break;
        case 'anual':
          query = `YEAR(data) = YEAR(CURDATE()) AND`
          console.log('Ação para anual');
          break;
        default:
          query = ``
          console.log('Ação para outro caso');
      }




      this.db.query(
        `SELECT 
              produtos.id AS id_produto, 
              produtos.nome, 
              produtos.descricao, 
              SUM(pedidos.quantidade) AS total_vendido
          FROM 
              produtos
          JOIN 
              pedidos ON pedidos.id_produto = produtos.id
          JOIN 
              user ON pedidos.id_user = user.id
          WHERE 
            ${query}
              pedidos.concluido = true 
              AND user.id = ?
          GROUP BY 
              produtos.id, produtos.nome, produtos.descricao
          ORDER BY 
              total_vendido DESC
          LIMIT 5`, [id_user],
        async (error, user) => {
          if (error) return reject(new Error(error));
          return resolve(user);
        }
      );
    });
  }

  /**
  * Lista os 5 produtos mais pedidos de um vendedor
  * @param {integer} id_vendedor 
  * @returns lista com os 5 produtos mais vendeidos de um vendedor
  */
  listTopProdutosVendedor(id_vendedor, periodo) {
    return new Promise(async (resolve, reject) => {

      let query = ``;
      switch (periodo) {
        case 'semanal':
          query = `YEARWEEK(pedidos.data, 1) = YEARWEEK(CURDATE(), 1) AND`
          console.log('Ação para semanal');
          break;
        case 'mensal':
          query = `YEAR(pedidos.data) = YEAR(CURDATE()) AND MONTH(pedidos.data) = MONTH(CURDATE()) AND`
          console.log('Ação para mensal');
          break;
        case 'anual':
          query = `YEAR(data) = YEAR(CURDATE()) AND`
          console.log('Ação para anual');
          break;
        default:
          query = ``
          console.log('Ação para outro caso');
      }




      this.db.query(
        `SELECT 
              produtos.id AS id_produto, 
              produtos.nome, 
              produtos.descricao, 
              SUM(pedidos.quantidade) AS total_vendido
          FROM 
              produtos
          JOIN 
              pedidos ON pedidos.id_produto = produtos.id
          JOIN 
              user ON pedidos.id_user = user.id
          JOIN
              vendedores ON user.id_vendedor = vendedores.id
          WHERE 
            
              pedidos.concluido = true 
              AND user.id_vendedor = ?
          GROUP BY 
              produtos.id, produtos.nome, produtos.descricao
          ORDER BY 
              total_vendido DESC
          LIMIT 5`, [id_vendedor],
        async (error, user) => {
          if (error) return reject(new Error(error));
          return resolve(user);
        }
      );
    });
  }

  /**
  * Lista os top 5 clientes que mais comprou produtos com o vendedor
  * @param {int} id_vendedor id do vendedor
  * @returns Os 5 clientes os clientes do vendedor 
  */
  listTopClientesVendedor(id_vendedor, periodo) {
    return new Promise(async (resolve, reject) => {

      if (!id_vendedor) return resolve({ error: `id do vendedor obrigatorio!`, code: 203 })

      let query = ``;
      switch (periodo) {
        case 'semanal':
          query = `YEARWEEK(pedidos.data, 1) = YEARWEEK(CURDATE(), 1) AND`
          console.log('Ação para semanal');
          break;
        case 'mensal':
          query = `YEAR(pedidos.data) = YEAR(CURDATE()) AND MONTH(pedidos.data) = MONTH(CURDATE()) AND`
          console.log('Ação para mensal');
          break;
        case 'anual':
          query = `YEAR(data) = YEAR(CURDATE()) AND`
          console.log('Ação para anual');
          break;
        default:
          query = ``
          console.log('Ação para outro caso');
      }



      this.db.query(
        `SELECT user.id, user.razao_social, user.documento, user.celular, user.email, user.role, user.status, user.tipo, user.endereco, user.qtde_minima_bovino, user.qtde_minima_suino,
            user.id_vendedor, user.anotacoes, DATE_FORMAT(user.data_criado, "%d/%m/%Y") AS data_criado, user.dias_pedido, user.aroma, 
            (
			        SELECT
				        SUM(pedidos.quantidade) AS qtde_caixas
                FROM pedidos
                WHERE
                pedidos.id_user = user.id 
            ) AS qtde_caixas
            FROM user, vendedores, pedidos, produtos
            WHERE
              ${query}
              pedidos.id_produto = produtos.id AND
              pedidos.concluido = TRUE AND
              pedidos.id_user = user.id AND
              user.id_vendedor = vendedores.id AND
              user.id_vendedor = ?
            GROUP BY user.id
            ORDER BY qtde_caixas DESC`,
        [id_vendedor],
        async (error, response) => {
          if (error) return reject(new Error(error));
          return resolve(response);
        }
      );
    });
  }


  listTopProdutosVendedor(id_vendedor) {
    return new Promise(async (resolve, reject) => {


      this.db.query(
        `SELECT 
              produtos.id AS id_produto, 
              produtos.nome, 
              produtos.descricao, 
              SUM(pedidos.quantidade) AS total_vendido
          FROM 
              produtos
          JOIN 
              pedidos ON pedidos.id_produto = produtos.id
          JOIN 
              user ON pedidos.id_user = user.id
          JOIN
              vendedores ON user.id_vendedor = vendedores.id
          WHERE 
              YEAR(pedidos.data) = YEAR(CURDATE()) AND MONTH(pedidos.data) = MONTH(CURDATE()) AND
              pedidos.concluido = true 
              AND user.id_vendedor = ?
          GROUP BY 
              produtos.id, produtos.nome, produtos.descricao
          ORDER BY 
              total_vendido DESC`, [id_vendedor],
        async (error, response) => {
          if (error) return reject(new Error(error));
          return resolve(response);
        }
      );
    });
  }


  listCountAllVendedorHTTP(id_vendedor, periodo) {
    return new Promise(async (resolve, reject) => {

      if (!id_vendedor) return resolve({ error: `ID do vendedor obrigatorio`, code: 203 })

      let query = ``;
      switch (periodo) {
        case 'semanal':
          query = `YEARWEEK(pedidos.data, 1) = YEARWEEK(CURDATE(), 1) AND`
          console.log('Ação para semanal');
          break;
        case 'mensal':
          query = `YEAR(pedidos.data) = YEAR(CURDATE()) AND MONTH(pedidos.data) = MONTH(CURDATE()) AND`
          console.log('Ação para mensal');
          break;
        case 'anual':
          query = `YEAR(data) = YEAR(CURDATE()) AND`
          console.log('Ação para anual');
          break;
        default:
          query = ``
          console.log('Ação para outro caso');
      }



      this.db.query(
        `SELECT 
  (SELECT COUNT(*) 
   FROM user 
   WHERE user.id_vendedor = ?) as clientes_vinculados,

  (SELECT SUM(pedidos.concluido) 
   FROM pedidos, user 
   WHERE 
       pedidos.id_user = user.id AND
       user.id_vendedor = ? AND
       pedidos.concluido = true
       ) AS pedidos_fechados,

  (SELECT SUM(pedidos.quantidade) AS qtde_caixa
   FROM pedidos, user 
   WHERE 
       pedidos.id_user = user.id AND
       pedidos.concluido = true AND
       user.id_vendedor = ? 
       ) as qtde_caixa,

  (SELECT AVG(mensal.pedidos_por_mes) 
   FROM (
       SELECT COUNT(*) AS pedidos_por_mes 
       FROM pedidos
       INNER JOIN user ON pedidos.id_user = user.id
       WHERE 
           pedidos.concluido = true AND 
           user.id_vendedor = ?
           
       GROUP BY EXTRACT(YEAR FROM pedidos.data), EXTRACT(MONTH FROM pedidos.data)
   ) AS mensal) AS media`,
        [id_vendedor, id_vendedor, id_vendedor, id_vendedor],
        (error, result) => {
          if (error) {
            console.error(error);
            return resolve({ error: "Erro ao consultar a quantidade total de pedidos.", code: 500 });
          }


          return resolve(result[0]);
        }
      );
    }
    )
  }


  /**
   * Lista a quantidade de vendas realizadas por mes de um vendedor
   * @param {Int} id_vendedor 
   * @returns Todas as quantidades de vendas realizadas separadas por mes
   */
  listVendasMensais(id_vendedor) {
    return new Promise(async (resolve, reject) => {

      if (!id_vendedor) return resolve({ error: `id do vendedor obrigatorio!`, code: 203 })


      this.db.query(
        `WITH recursive AllDates AS (
            SELECT DATE_FORMAT(CURDATE(), '%Y-01-01') AS data
            UNION ALL
            SELECT data + INTERVAL 1 MONTH
            FROM AllDates
            WHERE DATE_ADD(data, INTERVAL 1 DAY) <= DATE_FORMAT(CURDATE(), '%Y-12-31')
          )
          SELECT
              DATE_FORMAT(AllDates.data, "%d/%m") AS data,
               COALESCE(SUM(CASE WHEN user.id_vendedor = ? THEN pedidos.quantidade ELSE 0 END), 0) AS caixas
          FROM 
              AllDates
          LEFT JOIN 
              pedidos ON DATE(pedidos.data) = AllDates.data
          LEFT JOIN 
              user ON pedidos.id_user = user.id
			    LEFT JOIN 
              vendedores ON user.id_vendedor = vendedores.id
          WHERE 
              pedidos.concluido = 1 OR pedidos.concluido IS NULL
          GROUP BY 
              AllDates.data
          ORDER BY 
              AllDates.data
            Limit 366`,
        [id_vendedor],
        async (error, user) => {

          if (error) return reject(new Error(error));
          console.log(user);
          const dadosUnicos = [];
          const seen = new Set();


          const meses = [
            "janeiro", "fevereiro", "março", "abril", "maio", "junho",
            "julho", "agosto", "setembro", "outubro", "novembro", "dezembro"
          ];

          user.forEach(async (item) => {

            if (!seen.has(item.data)) {
              seen.add(item.data);


              const mesIndex = parseInt(item.data.split("/")[1], 10) - 1;
              item.data = meses[mesIndex];


            }
            dadosUnicos.push(item);

          });
          dadosUnicos.pop()
          return resolve(dadosUnicos)
        }
      );
    });
  }

  async uploadFileToS3(data, file) {

    const buffer = await sharp(file.buffer).resize({ fit: "contain" }).toBuffer()

    const command = new PutObjectCommand({
      Bucket: process.env.S3_BUCKET,
      Key: file.originalname,
      Body: buffer,
      ContentType: file.mimetype
    });

    try {
      const response = await this.s3Client.send(command);

      this.db.query(
        `INSERT INTO produtos (nome, descricao, image, linha, categoria, ordem, preco1, preco2, preco3, preco4, preco5) VALUES (?,?,?,?,?,?,?,?,?,?,?)`,
        [data.nome, data.descricao, file.originalname, data.linha, data.categoria, data.ordem, data.preco1, data.preco2, data.preco3, data.preco4, data.preco5],
        async (error, response) => {
          if (error) {
            console.error(error);
            return { error: "Erro ao cadastrar produto", code: 500 };
          }
          return { success: "produto adicionado com sucesso!" };
        }
      );

      return response;
    } catch (error) {
      console.error('Erro ao enviar para o S3:', error);
      return null;

    };
  }

  async getFileFromS3(id) {


    try {

      const produtos = await new Promise((resolve, reject) => {
        this.db.query(`SELECT * FROM produtos `, [id],
          async (error, produtos) => {

            if (error) {
              console.log(error);
              return reject(new Error(error));
            }

            resolve(produtos);
          });
      });

      for (const produto of produtos) {

        const ObjetoProduto = {
          Bucket: process.env.S3_BUCKET,
          Key: produto.image
        }

        // const command = new GetObjectCommand(ObjetoProduto)
        // const url = await getSignedUrl(this.s3Client, command)
        // const url = "https://brasapedidos.s3.us-east-2.amazonaws.com/"+produto.image
        // produto.image = url
      }
      return produtos
    } catch (error) {
      console.error('Erro ao enviar para o S3:', error);
      return null;

    };
  }


  listMetasMensalHTTP(id_vendedor, periodo) {
    return new Promise(async (resolve, reject) => {

      if (!db) return resolve({ error: "Banco de dados não configurado.", code: 203 });


      let query = ``;
      switch (periodo) {
        case 'semanal':
          query = `YEARWEEK(pedidos.data, 1) = YEARWEEK(CURDATE(), 1) AND`
          console.log('Ação para semanal');
          break;
        case 'mensal':
          query = `YEAR(pedidos.data) = YEAR(CURDATE()) AND MONTH(pedidos.data) = MONTH(CURDATE()) AND`
          console.log('Ação para mensal');
          break;
        case 'anual':
          query = `YEAR(data) = YEAR(CURDATE()) AND`
          console.log('Ação para anual');
          break;
        default:
          query = ``
          console.log('Ação para outro caso');
      }


      this.db.query(
        `SELECT 
          ( 
            SELECT COUNT(DISTINCT user.id) AS qtde_cliente_mensal
            FROM user, pedidos
            WHERE
            ${query}
            pedidos.id_user = user.id AND
            pedidos.concluido = true AND
            user.id_vendedor = ?

          ) AS qtde_cliente_mensal,
          (
            SELECT SUM(pedidos.quantidade) AS qtde_caixa
            FROM pedidos, user
            WHERE
            ${query}
            pedidos.id_user = user.id AND
            pedidos.concluido = true AND
            user.id_vendedor = ? 
          ) AS qtde_caixa_mensal,
          (
            SELECT SUM(pedidos.concluido) 
            FROM pedidos, user
            WHERE 
            ${query}
            pedidos.id_user = user.id AND
            user.id_vendedor = ? AND
            pedidos.concluido = true
          ) AS qtde_pedidos_mensal,
          (
            SELECT vendedores.meta_caixas_mensais
            FROM vendedores
            WHERE
            vendedores.id = ?
          ) AS meta_caixas_mensais,
          (
            SELECT vendedores.meta_pedidos_mensais
            FROM vendedores
            WHERE
            vendedores.id = ?
          ) AS meta_pedidos_mensais,
          (
            SELECT vendedores.meta_clientes_mensais
            FROM vendedores
            WHERE
            vendedores.id = ?
          ) AS meta_clientes_mensais
           `,
        [id_vendedor, id_vendedor, id_vendedor, id_vendedor, id_vendedor, id_vendedor],
        (error, result) => {
          if (error) {
            console.error(error);
            return resolve({ error: "Erro ao consultar a quantidade e metas mensais.", code: 500 });
          }
          return resolve(result[0]);
        }
      );
    }
    )
  }



  /**
   * Lista os dados de um usuario do vendedor
   * @param {Int} id_vendedor id o vendedor
   * @param {Int} id_user id o usuario
   * @returns lista com todos usuarios vinculado ao vendedor fornecido
   */
  listUser(id_user, id_vendedor) {
    return new Promise(async (resolve, reject) => {

      if (!id_vendedor) return resolve({ error: `ID do vendedor obrigatorio`, code: 203 })

      const query = id_user ? `user.id = ? AND` : "";


      try {
        this.db.query(
          `SELECT user.id, user.razao_social, user.documento, user.celular, user.email, user.role, user.status, 
          user.tipo, user.endereco, user.qtde_minima_suino, user.qtde_minima_bovino, user.id_vendedor, user.anotacoes, 
          DATE_FORMAT(user.data_criado, "%d/%m/%Y") AS data_criado,
          user.dias_pedido, user.aroma, user.meta_mensal, user.codUltra
          FROM user 
          WHERE
          ${query}
          user.id_vendedor = ? `,
          [id_vendedor, id_user ? id_user : ""],

          (error, usuarios) => {
            if (error) return reject(new Error(error));
            return resolve(usuarios);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  /**
   * 
   * @param {*} data 
   * @returns 
   */
  edit(data) {
    return new Promise(async (resolve, reject) => {

      const resultado = data.dias_pedido.join(' ');


      const values = [
        data.linha,
        data.qtde_minima_suino,
        data.qtde_minima_bovino,
        data.meta_mensal,
        resultado,
      ];


      values.push(data.id);

      this.db.query(
        `UPDATE user SET linha= ?, qtde_minima_suino = ?, qtde_minima_bovino = ?, meta_mensal=?, dias_pedido=? WHERE id = ?`,
        values,
        async (error, response) => {

          if (error) return reject(new Error(error));
          return resolve({ success: "Usuario atualizado com sucesso!", code: 200 });
        });
    });
  }

  /**
   * Lista os descontos de um usuario
   * @returns lista com todos os descontos de um usuario
   */
  listDescontosByUserVendedor(id_user) {
    return new Promise(async (resolve, reject) => {

      if (!id_user) return resolve({ error: `ID do usuario obrigatorio`, code: 203 })


      try {
        this.db.query(`SELECT 
                    produtos.id AS id_produto,
                    produtos.nome,
                    descontos.id AS id_desconto,
                    descontos.desconto,
                    descontos.operacao
                FROM 
                    produtos
                LEFT JOIN 
                    descontos ON descontos.id_produto = produtos.id AND descontos.id_user = ?`,
          [id_user ? id_user : ""],
          async (error, produtos) => {

            if (error) {

              console.log(error);
              return reject(new Error(error));
            }

            return resolve(produtos);
          });
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }


  listDashboard(data_inicio1, data_fim1, data_inicio2, data_fim2) {
    return new Promise(async (resolve, reject) => {
      try {
      const whereAtual = data_inicio1 && data_fim1 
      ? `BETWEEN '${data_inicio1}' AND '${data_fim1}'` 
      : `BETWEEN DATE_SUB(CURRENT_DATE, INTERVAL 1 MONTH) AND LAST_DAY(DATE_SUB(CURRENT_DATE, INTERVAL 1 MONTH))`;

    const whereAnterior = data_inicio2 && data_fim2 
      ? `BETWEEN '${data_inicio2}' AND '${data_fim2}'` 
      : `BETWEEN DATE_SUB(CURRENT_DATE, INTERVAL 2 MONTH) AND LAST_DAY(DATE_SUB(CURRENT_DATE, INTERVAL 2 MONTH))`;

    const dadosDashboard = await new Promise(async (resolve, reject) => {
      this.db.query(
        `SELECT 
          (SELECT COUNT(*) FROM vendedores) AS total_vendedores,
          (SELECT COUNT(*) FROM vendedores WHERE data_cadastro ${whereAtual}) AS vendedores_mes_atual,
          (SELECT COUNT(*) FROM vendedores WHERE data_cadastro ${whereAnterior}) AS vendedores_mes_anterior,
          (SELECT SUM(pedidos.quantidade) FROM pedidos) AS total_produtos,
          (SELECT SUM(pedidos.quantidade) FROM pedidos WHERE data ${whereAtual}) AS total_produtos_atual,
          (SELECT SUM(pedidos.quantidade) FROM pedidos WHERE data ${whereAnterior}) AS total_produtos_anterior,
          (SELECT COUNT(*) FROM user, vendedores WHERE user.id_vendedor = vendedores.id) AS total_clientes,
          (SELECT COUNT(*) FROM user, vendedores WHERE user.id_vendedor = vendedores.id AND data_criado ${whereAtual}) AS total_clientes_atual,
          (SELECT COUNT(*) FROM user, vendedores WHERE user.id_vendedor = vendedores.id AND data_criado ${whereAnterior}) AS total_clientes_anterior`,
            [],
            (error, results) => {
              if (error) {
                console.log(error);
                return reject(new Error(error));
              }
              return resolve(results[0]);
            }
          );
        });
        
        return resolve(dadosDashboard);
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }


  registerDesconto(dataArray) {
    return new Promise(async (resolve, reject) => {
      try {
        // Verificar se o parâmetro é um array
        if (!Array.isArray(dataArray)) {
          return resolve({ error: "Entrada deve ser um array de objetos.", code: 400 });
        }

        // Processar cada item no array
        for (const data of dataArray) {
          console.log(data);

          let descontoResults;

          // Verificar se o desconto já existe
          try {
            descontoResults = await new Promise((resolve, reject) => {
              this.db.query(
                `SELECT * FROM descontos WHERE id_user = ? AND id_produto = ?`,
                [data.id_user, data.id_produto],
                (error, results) => {
                  if (error) {
                    console.error("Erro ao buscar desconto existente:", error);
                    return reject(error);
                  }
                  resolve(results);
                }
              );
            });
          } catch (error) {
            console.error("Erro na consulta do banco de dados:", error);
            return resolve({ error: "Erro ao consultar descontos.", code: 500 });
          }

          console.log("Desconto existente:", descontoResults);

          // Verificar se o desconto já existe e atualizar ou inserir conforme necessário
          if (descontoResults.length > 0) {
            const existingOrder = descontoResults[0];
            const qtde = data.desconto

            console.log("Quantidade atualizada:", qtde);

            try {
              await new Promise((resolve, reject) => {
                this.db.query(
                  `UPDATE descontos SET desconto = ?, operacao = ? WHERE id_user = ? AND id_produto = ?`,
                  [qtde, data.operacao, data.id_user, data.id_produto],
                  (error) => {
                    if (error) {
                      console.error("Erro ao atualizar desconto:", error);
                      return reject(error);
                    }
                    resolve();
                  }
                );
              });
            } catch (error) {
              console.error("Erro ao atualizar desconto:", error);
              return resolve({ error: "Erro ao atualizar desconto.", code: 500 });
            }
          } else {
            console.log("Inserindo novo desconto");

            try {
              await new Promise((resolve, reject) => {
                this.db.query(
                  `INSERT INTO descontos (desconto, operacao, id_user, id_produto) VALUES (?, ?, ?, ?)`,
                  [data.desconto, data.operacao, data.id_user, data.id_produto],
                  (error) => {
                    if (error) {
                      console.error("Erro ao inserir desconto:", error);
                      return reject(error);
                    }
                    resolve();
                  }
                );
              });
            } catch (error) {
              console.error("Erro ao inserir desconto:", error);
              return resolve({ error: "Erro ao inserir desconto.", code: 500 });
            }
          }
        }

        // Retornar sucesso após processar todos os itens
        resolve({ success: "Descontos processados com sucesso!", code: 200 });
      } catch (error) {
        console.error("Erro ao processar descontos:", error);
        resolve({ error: "Erro interno do servidor.", code: 500 });
      }
    });
  }



  listFormVendedorByPeriodo(id_vendedor, periodo) {
    return new Promise(async (resolve, reject) => {

      if (!periodo) return resolve({ error: `periodo obrigatorio!`, code: 203 })
      if (!id_vendedor) return resolve({ error: `id do usuario obrigatorio!`, code: 203 })
      console.log(periodo);



      let query, query2, query3, query4 = ``
      switch (periodo) {
        case 'semanal':
          query = `DATE_SUB(CURDATE(), INTERVAL WEEKDAY(CURDATE()) DAY) AS data`
          query2 = `DATE_ADD(data, INTERVAL 1 DAY) <= DATE_ADD(DATE_SUB(CURDATE(), INTERVAL WEEKDAY(CURDATE()) DAY), INTERVAL 6 DAY)`
          query3 = `DAY`
          query4 = `"%d/%m"`

          break;
        case 'mensal':
          query = `DATE_FORMAT(DATE_SUB(CURDATE(), INTERVAL DAY(CURDATE()) - 1 DAY), '%Y-%m-%d') AS data`
          query2 = `DATE_ADD(data, INTERVAL 1 DAY) <= LAST_DAY(CURDATE())`
          query3 = `DAY`
          query4 = `"%d"`

          break;
        case 'anual':
          query = `DATE_FORMAT(CURDATE(), '%Y-01-01') AS data`
          query2 = `DATE_ADD(data, INTERVAL 1 DAY) <= DATE_FORMAT(CURDATE(), '%Y-12-31')`
          query3 = `MONTH`
          query4 = `"%d/%m"`

          break;
        default:
          query = ``
          query2 = ``
          query3 = ``
          query4 = ``


      }



      this.db.query(
        `WITH recursive AllDates AS (
          SELECT ${query}
          UNION ALL
          SELECT data + INTERVAL 1 ${query3} 
          FROM AllDates
          WHERE ${query2}
        )
        SELECT
    DATE_FORMAT(AllDates.data, '%d/%m') AS data, -- Formato dd/mm
    COALESCE(SUM(pedidos.quantidade), 0) AS caixas, -- Soma de caixas vendidas
    COALESCE(COUNT(pedidos.id), 0) AS pedidos -- Contagem de pedidos
FROM 
    AllDates
LEFT JOIN 
    pedidos 
    ON DATE(pedidos.data) = AllDates.data 
       AND pedidos.concluido = 1 -- Considera apenas pedidos concluídos
LEFT JOIN 
    user 
    ON pedidos.id_user = user.id 
       AND user.id_vendedor = ? 
GROUP BY 
    AllDates.data
ORDER BY 
    AllDates.data
LIMIT 366`,
        [id_vendedor],
        async (error, user) => {

          if (error) return reject(new Error(error));
          const dadosUnicos = [];
          const seen = new Set();


          const meses = [
            "janeiro", "fevereiro", "março", "abril", "maio", "junho",
            "julho", "agosto", "setembro", "outubro", "novembro", "dezembro"
          ];
          const dia_semana = [
            "domingo", "segunda", "terça", "quarta", "quinta", "sexta", "sabado", "sexta"
          ]

          user.forEach(async (item) => {

            if (!seen.has(item.data)) {
              seen.add(item.data);

              if (periodo === "anual") {
                const mesIndex = parseInt(item.data.split("/")[1], 10) - 1;
                item.data = meses[mesIndex];
              }

              if (periodo === "semanal") {
                const dataParts = item.data.split("/");
                const dia = dataParts[0];
                const mes = dataParts[1] - 1;

                const date = new Date(new Date().getFullYear(), mes, dia); // Usa o ano atual


                const diaIndex = date.getDay();


                item.data = dia_semana[diaIndex];
              }

            }
            dadosUnicos.push(item);

          });
          periodo === "anual" ? dadosUnicos.pop() : ``
          return resolve(dadosUnicos)
        }
      );
    });
  }

  registerUserPorVendedor(data, file) {
    return new Promise(async (resolve, reject) => {
      
    const resultado = 
    data.documento.length === 11 ? validarCPF(data.documento) :
    data.documento.length === 14 ? validarCNPJ(data.documento) : null;

    const celularValido = validarNumeroCelularLib(data.celular);

    if (!celularValido) return resolve({ error: "Celular Inválido!", code: 203 });
    if (!resultado) return resolve({ error: "CNPJ ou CPF inexistente!", code: 203 });

    const dateAtual = new Date();

    if (file) {
    const buffer = await sharp(file.buffer).resize({ fit: "contain" }).toBuffer()

    const command = new PutObjectCommand({
      Bucket: process.env.S3_BUCKET,
      Key: file.originalname,
      Body: buffer,
      ContentType: file.mimetype
    });

    await this.s3Client.send(command);
  }


      this.db.query(
        `INSERT INTO user (razao_social, documento, celular, email, role,  endereco, anotacoes, data_criado, notificacao_email, notificacao_whats, id_vendedor, image) VALUES(?,?,?,?,?,?,?,?,?,?,?,?)`,
        [data.razao_social, data.documento, data.celular, data.email, "USER",  data.endereco, data.anotacoes, dateAtual, data.notificacao_email, data.notificacao_whats, data.id_vendedor, file ? file.originalname : data.image],
        async (error, response) => {
          if (error) {
            console.error(error);
            resolve({ error: "Erro ao cadastrar usuario", code: 203 });
          }
          return resolve({ success: "Usuario cadastrado com sucesso!", code: 200 });
        })
    });
  }




  /**
   * Consulta quantidade de caixas de pedidos concluidos por usuario e dia fonercido
   * @param {*} dia quantidade de dia a ser consultado
   * @param {*} id_user ID do usuario a ser consultado
   * @returns quantidade de caixas e pedidos concluidos do usuario
   */
  listFormUsuarioProdutos(periodo, data_inicio1, data_fim1, data_inicio2, data_fim2, id_user) {
    return new Promise(async (resolve, reject) => {
        if (!id_user) return resolve({ error: `ID do usuário é obrigatório!`, code: 203 });

        let query = `
            SELECT 
                p.id AS id_produto, 
                p.nome, 
                DAYOFWEEK(pe.data) AS dia_semana, 
                COALESCE(SUM(CASE 
                                WHEN pe.data BETWEEN ? AND ? 
                                THEN pe.quantidade 
                            END), 0) AS quantidade_vendida_1,
                COALESCE(SUM(CASE 
                                WHEN pe.data BETWEEN ? AND ? 
                                THEN pe.quantidade 
                            END), 0) AS quantidade_vendida_2,
                COALESCE(SUM(CASE 
                                WHEN pe.data BETWEEN ? AND ? 
                                THEN pe.quantidade 
                            END), 0) - 
                COALESCE(SUM(CASE 
                                WHEN pe.data BETWEEN ? AND ? 
                                THEN pe.quantidade 
                            END), 0) AS diferenca
            FROM pedidos pe
            JOIN produtos p ON pe.id_produto = p.id
            WHERE pe.concluido = true
            AND pe.id_user = ?
            AND (pe.data BETWEEN ? AND ? 
                 OR pe.data BETWEEN ? AND ?)
            GROUP BY p.id, p.nome, dia_semana
            ORDER BY dia_semana, quantidade_vendida_1 DESC`;

        let params = [
            data_inicio1, data_fim1,
            data_inicio2, data_fim2,
            data_inicio1, data_fim1,
            data_inicio2, data_fim2,
            id_user,                
            data_inicio1, data_fim1,
            data_inicio2, data_fim2 
        ];

        console.log("QUERY SQL:", query);
        console.log("Parâmetros:", params);

        try {
            this.db.query(query, params, (error, resultados) => {
                if (error) return reject(new Error(error));
                return resolve(resultados);
            });
        } catch (error) {
            return reject(new Error(error));
        }
    });
}



}

module.exports = vendedoresRepository;
